<?php

/**
 * @file
 * Definition of Drupal\maestro\Plugin\views\field\MaestroEngineActiveHandler
 */

namespace Drupal\maestro\Plugin\views\field;

use Drupal\Core\Form\FormStateInterface;
use Drupal\maestro\Engine\MaestroEngine;
use Drupal\maestro\Utility\TaskHandler;
use Drupal\views\Plugin\views\field\FieldPluginBase;
use Drupal\views\ResultRow;
use Drupal\Core\Url;
use Drupal\Component\Serialization\Json;
use Drupal\Component\Utility\UrlHelper;

/**
 * Field handler to create a usable link to the task via the handler field
 *
 * @ingroup views_field_handlers
 *
 * @ViewsField("maestro_active_handler")
 */
class MaestroEngineActiveHandler extends FieldPluginBase {

  /**
   * @{inheritdoc}
   */
  public function query() {
    // no Query to be done.
  }

  /**
   * Define the available options
   * @return array
   */
  protected function defineOptions() {
    $options = parent::defineOptions();


    return $options;
  }

  /**
   * Provide the options form.
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {


    parent::buildOptionsForm($form, $form_state);
  }

  /**
   * @{inheritdoc}
   */
  public function render(ResultRow $values) {
    global $base_url;

    $item = $values->_entity;
    //this will ONLY work for production assignments and/or maestro queues
    if ($item->getEntityTypeId() == 'maestro_production_assignments' || $item->getEntityTypeId() == 'maestro_queue') {
      //we are of the right types.  So let's get the right queue ID
      if ($item->getEntityTypeId() == 'maestro_production_assignments') {
        $queueID = $item->queue_id->getString();
      }
      else {
        $queueID = $item->id->getString();
      }
      $engine = new MaestroEngine();

      //ok, so at this point, we are going to try to pick off the active handler
      $queueRecord = MaestroEngine::getQueueEntryById($queueID);
      /*
       * Logic to handle all known types of interactive task types and
       * handlers
       */
      $handler = $queueRecord->handler->getString();

        if (!empty($handler)) {
          /*
           * Strip out the base url from the handler URL and build a new URL
           * Using the Drupal API's to properly filter and format it
           * depending on the handler type (external,internal,function or unknown)
           */

          $handler = str_replace($base_url, '', $handler);
          $handler_type = TaskHandler::getType($handler);

          $templateMachineName = $engine->getTemplateIdFromProcessId($queueRecord->process_id->getString());
          $taskTemplate = $engine->getTemplateTaskByID($templateMachineName, $queueRecord->task_id->getString());

          $query_options = array('queueid' => $queueID);
          $handler_url_parts = UrlHelper::parse($handler);
          $query_options += $handler_url_parts['query'];

          $use_modal = FALSE;
          if (array_key_exists('data', $taskTemplate) && array_key_exists('modal', $taskTemplate['data']) && $taskTemplate['data']['modal'] == 'modal') {
            $use_modal = TRUE;
          }

          switch($handler_type) {
            case 'external' :
              $form['task_console_table'][$queueID]['execute'] = array(
                '#type' => 'link',
                '#title' => $this->t('Link'),
                '#url' => Url::fromUri($handler, array('query' => $query_options)),
              );
              break;
            case 'internal':
              $form['task_console_table'][$queueID]['execute'] = array(
                '#type' => 'link',
                '#title' => $this->t('Link'),
                '#url' => Url::fromUserInput($handler, array('query' => $query_options)),
              );
              break;

            case 'function':
              if ($use_modal) {
                $query_options += ['modal' => 'modal'];
                $form['task_console_table'][$queueID]['execute'] = array(
                  '#title' => $this->t('Link'),
                  '#type' => 'link',
                  '#url' => Url::fromRoute('maestro.execute', $query_options),
                  '#attributes' => [
                    'class' => ['use-ajax'],
                    'data-dialog-type' => 'modal',
                    'data-dialog-options' => Json::encode([
                      'width' => 700,
                    ]),
                  ],
                );
              }
              else {
                $query_options += ['modal' => 'notmodal'];
                $form['task_console_table'][$queueID]['execute'] = array(
                  '#title' => $this->t('Link'),
                  '#type' => 'link',
                  '#url' => Url::fromRoute('maestro.execute', $query_options),
                );
              }
              break;

            default:
              $form['task_console_table'][$queueID]['execute'] = array(
                '#plain_text' => $this->t('Invalid Link'),
              );
          }

        }
        else { //we don't have a valid URL here. It's actually blank.  Can't create a link to nothing.
          $form['task_console_table'][$queueID]['execute'] = array(
            '#plain_text' => $this->t('No Handler'),
          );
        }

      return $form;
    }
    else {
      return '';
    }
  }
}