<?php

namespace Drupal\maestro\Utility;

use Drupal\Core\Url;
use Drupal\Component\Utility\UrlHelper;

/*
 * Internal function that helps determine the handler type that is
 * used for interactive tasks
 *
 * Returns either unknown, external, internal or function
 */
class TaskHandler {

  public static function getType($handler) {
    global $base_url;

    $handler = str_replace($base_url, '', $handler);

    $handler_type = 'unknown';
    $is_externalRoute = UrlHelper::isExternal($handler);
    if ($is_externalRoute === FALSE) {
      try {
        if (function_exists($handler)) {
          $handler_type = 'function';
        }
        else {
          $url = Url::fromUri("internal:" . $handler);
          if ($url && $url->isRouted()) {
            $handler_type = 'internal';
          }
        }
      } catch (\InvalidArgumentException $e) {
        $handler_type = 'unknown';
      }
    }
    else {
      $handler_type = 'external';
    }

    return $handler_type;
  }

}